@@ -6,9 +6,9 @@ from msgpack import ExtType
 from .buffer import Buffer
 from .common import (DecodeHook, Remote, RemoteMap, RemoteSequence,
                      SessionFilter, SessionHook, walk)
-from ..compat import IS_PYTHON3
 from .tabpage import Tabpage
 from .window import Window
+from ..compat import IS_PYTHON3
 
 
 __all__ = ('Nvim')
@@ -44,7 +44,10 @@ class Nvim(object):
         creating specialized objects from Nvim remote handles.
         """
         session.error_wrapper = lambda e: NvimError(e[1])
-        channel_id, metadata = session.request('vim_get_api_info')
+        channel_id, metadata = session.request(b'vim_get_api_info')
+
+        encoding = session.request(b'vim_get_option', b'encoding')
+        session._async_session._msgpack_stream.set_packer_encoding(encoding)
 
         if IS_PYTHON3:
             hook = DecodeHook()
@@ -126,7 +129,7 @@ class Nvim(object):
             try:
                 if cb(path) is not None:
                     break
-            except:
+            except Exception:
                 break
 
     def chdir(self, dir_path):
@@ -196,6 +199,7 @@ class Current(object):
 
     def __init__(self, session):
         self._session = session
+        self.range = None
 
     @property
     def line(self):
