@@ -2,12 +2,15 @@ import imp
 import logging
 import sys
 
-from ..api.common import SessionHook
+from ..api.common import SessionHook, DecodeHook
+from ..compat import NUM_TYPES, IS_PYTHON3
 
 
 logger = logging.getLogger(__name__)
 debug, warn = (logger.debug, logger.warn,)
 
+if IS_PYTHON3:
+    basestring = str
 
 class ScriptHost(object):
     """
@@ -23,15 +26,24 @@ class ScriptHost(object):
         # it seems some plugins assume 'sys' is already imported, so do it now
         exec('import sys', self.module.__dict__)
         sys.modules['vim'] = nvim.with_hook(LegacyEvalHook())
+        if IS_PYTHON3:
+            sys.modules['vim'] = sys.modules['vim'].with_hook(DecodeHook(encoding=nvim.options['encoding'].decode('ascii')))
+        self.legacy_vim = sys.modules['vim']
 
-    def python_execute(self, script):
+    def python_execute(self, script, range_start, range_end):
+        self._set_current_range(range_start, range_end)
         exec(script, self.module.__dict__)
 
-    def python_execute_file(self, file_path):
+    def python_execute_file(self, file_path, range_start, range_end):
+        self._set_current_range(range_start, range_end)
         with open(file_path) as f:
             script = compile(f.read(), file_path, 'exec')
             exec(script, self.module.__dict__)
 
+    def _set_current_range(self, range_start, range_end):
+        self.legacy_vim.current.range = \
+                self.legacy_vim.current.buffer.range(range_start, range_end)
+
     def python_do_range(self, start, stop, code):
         nvim = self.nvim
         start -= 1
@@ -55,13 +67,31 @@ class ScriptHost(object):
             sstart = start
             sstop = min(start + 5000, stop)
             lines = nvim.current.buffer.get_line_slice(sstart, sstop, True, True)
+
+            exception = None
+            newlines = []
+            linenr = sstart + 1
             for i, line in enumerate(lines):
-                linenr = i + sstart + 1
-                result = str(function(line, linenr))
-                if result:
-                    lines[i] = result
+                result = function(line, linenr)
+                if result == None:
+                    # Update earlier lines, and skip to the next
+                    if newlines:
+                        nvim.current.buffer.set_line_slice(sstart, sstart + len(newlines) -1, True, True, newlines)
+                    sstart += len(newlines) + 1
+                    newlines = []
+                    pass
+                elif isinstance(result,basestring):
+                    newlines.append(result)
+                else:
+                    exception = TypeError('pydo should return a string or None, found %s instead' % result.__class__.__name__)
+                    break
+                linenr += 1
+
             start = sstop + 1
-            nvim.current.buffer.set_line_slice(sstart, sstop, True, True, lines)
+            if newlines:
+                nvim.current.buffer.set_line_slice(sstart, sstart + len(newlines)-1, True, True, newlines)
+            if exception:
+                raise exception
         # delete the function
         del self.module.__dict__[fname]
 
@@ -77,6 +107,6 @@ class LegacyEvalHook(SessionHook):
         super(LegacyEvalHook, self).__init__(from_nvim=self._string_eval)
 
     def _string_eval(self, obj, session, method, kind):
-        if method == 'vim_eval' and isinstance(obj, (int, long, float)):
+        if method == 'vim_eval' and isinstance(obj, NUM_TYPES):
             return str(obj)
         return obj
