@@ -2,7 +2,7 @@ from time import sleep
 from random import random
 from nose.tools import with_setup, eq_ as eq
 from common import vim, cleanup
-from threading import Thread
+from threading import Thread, Timer
 
 
 @with_setup(setup=cleanup)
@@ -22,3 +22,12 @@ def test_custom_messages():
         custom_messages.append(vim.session.next_message())
 
     eq(len(custom_messages), 50)
+
+
+@with_setup(setup=cleanup)
+def test_interrupt_from_another_thread():
+    timer = Timer(0.5, lambda: vim.session.post('timeout'))
+    timer.start()
+    msg = vim.session.next_message()
+    eq(msg[0], 'notification')
+    eq(msg[1], 'timeout')
