@@ -44,6 +44,14 @@ def test_python_nested_commands():
     eq(vim.vars['set_by_nested_python'], 555)
 
 
+@with_setup(setup=host_setup, teardown=host_teardown)
+def test_python_command_with_range():
+    vim.feedkeys('iline1\nline2\nline3\nline4\033')
+    vim.feedkeys('ggjvj:python vim.vars["range"] = vim.current.range[:]\n')
+    vim.eval('1') # wait for the keys to be processed
+    eq(vim.vars['range'], ['line2', 'line3'])
+
+
 @with_setup(setup=host_setup, teardown=host_teardown)
 def test_pyfile():
     fname = 'pyfile.py'
@@ -56,11 +64,17 @@ def test_pyfile():
 
 @with_setup(setup=host_setup, teardown=host_teardown)
 def test_pydo():
+
+    # :pydo 42 returns None for all lines,
+    # the buffer should not be changed
+    vim.command('normal :pydo 42\n')
+    eq(vim.current.buffer.options['mod'], False)
+
     # insert some text
     vim.command('normal iabc\ndef\nghi')
     eq(vim.current.buffer[:], ['abc', 'def', 'ghi'])
     # go to top and select and replace the first two lines
-    vim.command('normal ggvj:pydo return linenr\n')
+    vim.command('normal ggvj:pydo return str(linenr)\n')
     eq(vim.current.buffer[:], ['1', '2', 'ghi'])
 
 
